"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.typedocPlugin = void 0;
const path = require("path");
const typedoc_1 = require("typedoc");
const front_matter_1 = require("typedoc-plugin-markdown/dist/components/front-matter");
const sidebar_1 = require("./sidebar");
const DEFAULT_PLUGIN_OPTIONS = {
    out: 'api',
    sidebar: {
        parentCategory: 'none',
        fullNames: false,
    },
};
const app = new typedoc_1.Application();
let done = false;
exports.typedocPlugin = async (opts, ctx) => {
    const sourceDir = ctx.sourceDir;
    const options = { ...DEFAULT_PLUGIN_OPTIONS, ...opts };
    const outFolder = options.out ? options.out : 'api';
    const out = sourceDir + '/' + outFolder;
    const sidebar = options.sidebar;
    if (done) {
        return;
    }
    const vuepressOptions = Object.keys(options).reduce((option, key) => {
        if (![...['id'], ...Object.keys(DEFAULT_PLUGIN_OPTIONS)].includes(key)) {
            option[key] = options[key];
        }
        return option;
    }, {});
    app.options.addReader(new typedoc_1.TypeDocReader());
    app.options.addReader(new typedoc_1.TSConfigReader());
    app.bootstrap({
        ...vuepressOptions,
        plugin: [
            ...['typedoc-plugin-markdown'],
            ...(opts.plugin
                ? opts.plugin.filter((name) => name !== 'typedoc-plugin-markdown')
                : []),
        ],
        theme: path.resolve(__dirname, 'theme'),
    });
    app.renderer.addComponent('frontmatter', new front_matter_1.FrontMatterComponent(app.renderer));
    const project = app.convert();
    if (!project) {
        done = true;
        return;
    }
    await app.generateDocs(project, out);
    return {
        name: 'vuepress-plugin-typedoc',
        async enhanceAppFiles() {
            if (done || !sidebar) {
                return;
            }
            const theme = app.renderer.theme;
            const navigation = theme.getNavigation(project);
            return {
                name: 'typedoc-sidebar',
                content: `export default ({ siteData, options }) => {
          siteData.themeConfig.sidebarDepth = 2;
          siteData.themeConfig.sidebar = Object.assign({},siteData.themeConfig.sidebar,${JSON.stringify({
                    [`/${outFolder}/`]: sidebar_1.getSidebarJson(navigation, outFolder, sidebar),
                })});
        }`,
            };
        },
        afterDevServer() {
            done = true;
        },
    };
};
